import numpy as np
import pandas as pd

results_r = pd.read_csv("results_r.csv")
results_py = pd.read_csv("results_python.csv")
results = pd.merge(results_r, results_py, on="file", suffixes=("_r", "_py"))
results["diff"] = results["rho_r"] - results["rho_py"]

tol = 0.01
results["is_close"] = np.isclose(results["rho_r"], results["rho_py"], atol=tol)
n_close = results["is_close"].sum()


print(
    f"""
# ----------------------------------------
# Point Biserial
# ----------------------------------------
R vs Python results:

{results}

{n_close} out of {len(results)} results are close (within {tol}) to each other.
"""
)



import pandas as pd
from pathlib import Path

from ordinalcorr import polychoric
from semopy.polycorr import polychoric_corr as semopy_corr


data_dir = Path("./data")
csv_paths = data_dir.glob("*.csv")

results = []
for csv_path in sorted(csv_paths):
    df = pd.read_csv(csv_path)
    rho = polychoric(df["x1"], df["x2"])
    rho_semopy = semopy_corr(df["x1"], df["x2"])
    results.append({"file": csv_path.name, "rho": rho, "rho_semopy": rho_semopy})
    # print(f"{csv_path.name}: {rho:.4f}")

pd.DataFrame(results).to_csv("results_python.csv", index=False)



# Configuration file for the Sphinx documentation builder.
#
# For the full list of built-in configuration values, see the documentation:
# https://www.sphinx-doc.org/en/master/usage/configuration.html

# -- Project information -----------------------------------------------------
# https://www.sphinx-doc.org/en/master/usage/configuration.html#project-information

project = "ordinalcorr"
copyright = "2025, Masayoshi Mita"
author = "Masayoshi Mita"

# -- General configuration ---------------------------------------------------
# https://www.sphinx-doc.org/en/master/usage/configuration.html#general-configuration

extensions = [
    "sphinx.ext.autodoc",
    "sphinx.ext.autosummary",
    # "sphinx.ext.doctest",
    "sphinx.ext.napoleon",
    "sphinx.ext.viewcode",
    "sphinx.ext.githubpages",
    "myst_nb",
]

myst_enable_extensions = [
    "dollarmath",
]

templates_path = ["_templates"]
exclude_patterns = ["_build", "Thumbs.db", ".DS_Store"]


# -- Options for HTML output -------------------------------------------------
# https://www.sphinx-doc.org/en/master/usage/configuration.html#options-for-html-output

html_theme = "furo"
html_static_path = ["_static"]


import sys
from pathlib import Path

sys.path.insert(0, str(Path("..").resolve()))
autodoc_default_options = {"members": True}



import pandas as pd
import numpy as np
from ordinalcorr.polytomous import polychoric, polyserial


def hetcor(data: pd.DataFrame, n_unique: int = 20) -> pd.DataFrame:
    """
    Estimate the heterogeneous correlation matrix.

    The heterogeneous correlation matrix includes:

    - Pearson product-moment correlations between continuous variables
    - Polychoric correlations between ordinal variables
    - Polyserial correlations between continuous and ordinal variables

    Parameters
    ----------
    data : pd.DataFrame
        A DataFrame containing continuous and/or ordinal variables.
        Appropriate correlation coefficients are automatically selected based on the types of variables.

        - Columns with dtype `float` are treated as continuous variables.
        - Columns with dtype `int` and number of unique values less than or equal to `n_unique`
          are treated as ordinal variables.
        - Columns with dtype `category` are treated as ordinal variables if they are ordered.

    n_unique : int, default=20
        The maximum number of unique values for an integer column to be considered ordinal.
        If the number of unique values exceeds `n_unique`, the column is treated as continuous.


    Returns
    -------
    pd.DataFrame
        Estimated heterogeneous correlation matrix.


    Examples
    --------
    >>> from ordinalcorr import hetcor
    >>> import pandas as pd
    >>> data = pd.DataFrame({
    ...     "continuous": [0.1, 0.1, 0.2, 0.2, 0.3, 0.3],
    ...     "ordinal": [0, 0, 0, 1, 1, 2],
    ... })
    >>> hetcor(data)
    """
    # NOTE: np.ndarray uses single dtype, so it cannot be a input. so we accept pd.DataFrame only
    if not isinstance(data, pd.DataFrame):
        raise TypeError("Input data must be a pandas.DataFrame")

    is_col_ordinal = is_cols_ordinal(data, n_unique=n_unique)

    ncols = len(data.columns)
    corr = np.zeros((ncols, ncols), dtype=float)
    for i in range(ncols):
        for j in range(i, ncols):
            if i == j:
                corr[i, j] = 1.0
                continue

            if is_col_ordinal[i] and is_col_ordinal[j]:
                corr[i, j] = polychoric(data.iloc[:, i], data.iloc[:, j])

            if is_col_ordinal[i] and not is_col_ordinal[j]:
                ordinal = data.iloc[:, i]
                continuous = data.iloc[:, j]
                corr[i, j] = polyserial(continuous, ordinal)

            if not is_col_ordinal[i] and is_col_ordinal[j]:
                continuous = data.iloc[:, i]
                ordinal = data.iloc[:, j]
                corr[i, j] = polyserial(continuous, ordinal)

            if not is_col_ordinal[i] and not is_col_ordinal[j]:
                from scipy.stats import pearsonr

                corr[i, j] = pearsonr(data.iloc[:, i], data.iloc[:, j]).statistic

            corr[j, i] = corr[i, j]
    corr_df = pd.DataFrame(corr, index=data.columns, columns=data.columns)
    return corr_df


def is_cols_ordinal(data: pd.DataFrame, n_unique: int) -> list[bool]:
    """Check if the columns of the DataFrame are ordinal."""
    results = [None] * len(data.columns)
    for j in range(len(data.columns)):
        results[j] = is_col_ordinal(x=data.iloc[:, j], n_unique=n_unique)
    return results


def is_col_ordinal(x: pd.Series, n_unique: int) -> bool:
    """Check if the input is ordinal."""
    if "int" in x.dtype.name.lower() and x.unique().size <= n_unique:
        return True

    if x.dtype.name == "category":
        if x.cat.ordered:
            return True
        raise TypeError(f"The column '{x.name}' is unoredered category.")

    return False



import warnings
import numpy as np
from scipy.stats import norm, multivariate_normal
from scipy.optimize import minimize_scalar
from ordinalcorr.validation import (
    ValidationError,
    check_if_zero_variance,
    check_length_are_same,
)
from ordinalcorr.types import ArrayLike


def univariate_cdf(lower, upper):
    """Compute the univariate cumulative distribution function (CDF) for a standard normal distribution.

    P(lower < X <= upper) = Φ(upper) - Φ(lower)

    where Φ is the CDF of the standard normal distribution.
    """
    mean = 0.0
    var = 1.0
    std = np.sqrt(var)
    return norm.cdf(upper, loc=mean, scale=std) - norm.cdf(lower, loc=mean, scale=std)


def bivariate_cdf(lower, upper, rho: float) -> float:
    """Compute the bivariate cumulative distribution function (CDF) for a standard normal distribution.

    P(lower_x < X <= upper_x, lower_y < Y <= upper_y)
        = Φ₂(upper_x, upper_y) - Φ₂(lower_x, upper_y) - Φ₂(upper_x, lower_y) + Φ₂(lower_x, lower_y)

    where Φ₂ is the CDF of the bivariate normal distribution with correlation coefficient ρ.
    """
    var = 1
    cov = np.array([[var, rho], [rho, var]])
    Phi2 = multivariate_normal(mean=[0, 0], cov=cov, allow_singular=True).cdf
    return (
        Phi2(upper)
        - Phi2([upper[0], lower[1]])
        - Phi2([lower[0], upper[1]])
        + Phi2(lower)
    )


def estimate_thresholds(values):
    """Estimate thresholds from empirical marginal proportions"""
    inf = 100  # to make log-likelihood smooth, use large value instead of np.inf
    thresholds = []
    levels = np.sort(np.unique(values))
    for level in levels[:-1]:  # exclude top category
        p = np.mean(values <= level)
        thresholds.append(norm.ppf(p))  # τ_i = Φ⁻¹(P(X ≤ i))
    return np.concatenate(([-inf], thresholds, [inf]))


def normalize_ordinal(x: np.ndarray[int]) -> np.ndarray[int]:
    """Normalize ordinal variable to be integer-coded starting from 0."""
    unique_values = np.unique(x)
    value_to_code = {value: code for code, value in enumerate(unique_values)}
    return np.vectorize(value_to_code.get)(x)


def polychoric(x: ArrayLike[int], y: ArrayLike[int]) -> float:
    """
    Estimate the polychoric correlation coefficient between two ordinal variables.

    The polychoric correlation assumes that the two observed ordinal variables
    are thresholded representations of underlying continuous variables that follow
    a bivariate normal distribution.


    Parameters
    ----------
    x : array_like (int)
        Ordinal variable.
    y : array_like (int)
        Ordinal variable.

    Returns
    -------
    float
        Estimated polychoric correlation coefficient.

    Examples
    --------
    >>> from ordinalcorr import polychoric
    >>> x = [1, 1, 2, 2, 3, 3]
    >>> y = [0, 0, 0, 1, 1, 1]
    >>> polychoric(x, y)

    References
    ----------
    .. [1] Olsson, U. (1979). Maximum likelihood estimation of the polychoric correlation coefficient. Psychometrika, 44(4), 443-460.
    .. [2] Drasgow, F. (1986). Polychoric and polyserial correlations In: Kotz S, Johnson N, editors. The Encyclopedia of Statistics.
    """
    x = np.asarray(x)
    y = np.asarray(y)

    try:
        check_length_are_same(x, y)
        check_if_zero_variance(x)
        check_if_zero_variance(y)
    except ValidationError as e:
        warnings.warn(str(e))
        return np.nan

    x_levels = np.sort(np.unique(x))
    y_levels = np.sort(np.unique(y))

    if x_levels.size <= 1 or y_levels.size <= 1:
        Warning("Both x and y must have at least two unique ordinal levels.")
        return np.nan

    tau_x = estimate_thresholds(x)  # thresholds for X: τ_X
    tau_y = estimate_thresholds(y)  # thresholds for Y: τ_Y

    contingency = np.zeros((len(tau_x) - 1, len(tau_y) - 1), dtype=int)
    for i, xi in enumerate(x_levels):
        for j, yj in enumerate(y_levels):
            contingency[i, j] = np.sum((x == xi) & (y == yj))  # n_ij

    def neg_log_likelihood(rho):
        log_likelihood = 0.0
        for i in range(len(tau_x) - 1):
            for j in range(len(tau_y) - 1):
                if contingency[i, j] == 0:
                    continue

                lower = [tau_x[i], tau_y[j]]
                upper = [tau_x[i + 1], tau_y[j + 1]]

                p_ij = bivariate_cdf(lower, upper, rho)
                p_ij = max(p_ij, 1e-6)  # soft clipping

                if np.isnan(p_ij):
                    continue

                log_likelihood += contingency[i, j] * np.log(p_ij)

        return -log_likelihood

    result = minimize_scalar(neg_log_likelihood, bounds=(-1, 1), method="bounded")
    return result.x


def polyserial(x: ArrayLike[float | int], y: ArrayLike[int]) -> float:
    """
    Estimate the polyserial correlation coefficient between a continuous variable x
    and an ordinal variable y using the two-step maximum likelihood estimation.

    The polyserial correlation assumes that the ordinal variable y is a thresholded
    representation of latent continuous variable that follows a normal distribution.


    Parameters
    ----------
    x : array_like (float | int)
        Continuous variable.
    y : array_like (int)
        Ordinal variable.

    Returns
    -------
    float
        Estimated polyserial correlation coefficient.

    Examples
    --------
    >>> from ordinalcorr import polyserial
    >>> x = [0.1, 0.1, 0.2, 0.2, 0.3, 0.3]
    >>> y = [0, 0, 0, 1, 1, 2]
    >>> polyserial(x, y)

    References
    ----------
    .. [1] Drasgow, F. (1986). Polychoric and polyserial correlations In: Kotz S, Johnson N, editors. The Encyclopedia of Statistics.
    """
    x = np.asarray(x)
    y = np.asarray(y)

    try:
        check_length_are_same(x, y)
        check_if_zero_variance(x)
        check_if_zero_variance(y)
    except ValidationError as e:
        warnings.warn(str(e))
        return np.nan

    z = (x - np.mean(x)) / np.std(x, ddof=0)
    y = normalize_ordinal(y)
    tau = estimate_thresholds(y)

    def neg_log_likelihood(rho):
        log_likelihood = 0.0
        for i in range(len(z)):
            j = y[i]
            tau_lower = (tau[j] - rho * z[i]) / np.sqrt(1 - rho**2)
            tau_upper = (tau[j + 1] - rho * z[i]) / np.sqrt(1 - rho**2)
            p_i = univariate_cdf(tau_lower, tau_upper)

            p_i = max(p_i, 1e-6)  # soft clipping
            if np.isnan(p_i):
                continue

            log_likelihood += np.log(p_i)

        return -log_likelihood

    result = minimize_scalar(neg_log_likelihood, bounds=(-1, 1), method="bounded")
    return result.x



import warnings
import numpy as np
from scipy.stats import norm, multivariate_normal
from scipy.optimize import minimize_scalar
from ordinalcorr.types import ArrayLike
from ordinalcorr.validation import (
    ValidationError,
    check_if_data_is_dichotomous,
    check_if_zero_variance,
    check_length_are_same,
)


def biserial(x: ArrayLike[float | int], y: ArrayLike[int]) -> float:
    """
    Compute the biserial correlation coefficient between a continuous variable x
    and a dichotomized variable y (0 or 1), assuming y was split from a latent continuous variable.

    Parameters
    ----------
    x : array-like
        Continuous variable.
    y : array-like
        Dichotomous variable (0 and 1), assumed to be derived from a latent continuous variable.

    Returns
    -------
    float
        Biserial correlation coefficient.

    Examples
    --------
    >>> from ordinalcorr import biserial
    >>> x = [0.1, 0.2, 0.3, 0.4, 0.5]
    >>> y = [0, 0, 1, 1, 1]
    >>> biserial(x, y)

    :Details:

    The biserial correlation coefficient is defined as:

    .. math::

        r_{b} = r_{pb} \\cdot \\frac{\\sqrt{p (1 - p)}}{\\phi(z)}
        = \\frac{\\bar{X}_1 - \\bar{X}_0}{s_X} \\cdot \\frac{p (1 - p)}{\\phi(z)}

    where

    - :math:`r_{pb}` is the point-biserial correlation coefficient
    - :math:`\\bar{X}_1` and :math:`\\bar{X}_0` are the means of the continuous variable for the two categories of the dichotomous variable
        - :math:`\\bar{X}_1 = \\frac{1}{n_1} \\sum_{i:Y_i = 1} X_i, \\quad n_1 = |\\{i: Y_i = 1\\}|`
        - :math:`\\bar{X}_0 = \\frac{1}{n_0} \\sum_{i:Y_i = 0} X_i, \\quad n_0 = |\\{i: Y_i = 0\\}|`
    - :math:`s_X` is the standard deviation of the continuous variable
        - :math:`s_X = \\sqrt{\\frac{1}{n} \\sum_{i=1}^{n} (X_i - \\bar{X})^2}, \\quad n = n_1 + n_0`
    - :math:`p` is the proportion of `Y = 1` of the dichotomous variable
        - :math:`p = \\frac{n_1}{n}`
    - :math:`\\phi(z)` is the probability density function of the standard normal distribution
    - :math:`z` is the percentile of :math:`p` in the standard normal distribution


    """
    rho_pbi = point_biserial(x, y)
    if np.isnan(rho_pbi):
        return np.nan

    # calculate the correction factor
    p = np.mean(y)
    q = 1 - p
    z = norm.ppf(p)
    c = np.sqrt(p * q) / norm.pdf(z)
    return rho_pbi * c


def point_biserial(x: ArrayLike, y: ArrayLike) -> float:
    """
    Compute the point-biserial correlation between a continuous variable x
    and a dichotomous variable y (0 or 1), assuming y is a true dichotomous variable.

    Parameters
    ----------
    x : array-like
        Continuous variable.
    y : array-like
        Dichotomous variable (0 and 1).


    Returns
    -------
    float
        Point-biserial correlation coefficient.


    Examples
    --------
    >>> from ordinalcorr import point_biserial
    >>> x = [0.1, 0.2, 0.3, 0.4, 0.5]
    >>> y = [0, 0, 1, 1, 1]
    >>> point_biserial(x, y)


    :Details:

    The point-biserial correlation coefficient is defined as:

    .. math::

        r_{pb} = \\frac{\\bar{X}_1 - \\bar{X}_0}{s_X} \\sqrt{p \\times (1 - p)}

    where

    - :math:`\\bar{X}_1` and :math:`\\bar{X}_0` are the means of the continuous variable for the two categories of the dichotomous variable
        - :math:`\\bar{X}_1 = \\frac{1}{n_1} \\sum_{i:Y_i = 1} X_i, \\quad n_1 = |\\{i: Y_i = 1\\}|`
        - :math:`\\bar{X}_0 = \\frac{1}{n_0} \\sum_{i:Y_i = 0} X_i, \\quad n_0 = |\\{i: Y_i = 0\\}|`
    - :math:`s_X` is the standard deviation of the continuous variable
        - :math:`s_X = \\sqrt{\\frac{1}{n} \\sum_{i=1}^{n} (X_i - \\bar{X})^2}, \\quad n = n_1 + n_0`
    - :math:`p` is the proportion of `Y = 1` of the dichotomous variable
        - :math:`p = \\frac{n_1}{n}`

    Note that the point-biserial correlation coefficient is equivalent to the Pearson correlation coefficient between the continuous variable and the dichotomous variable.

    References
    ----------
    .. [1] Lev, J. (1949). The point biserial coefficient of correlation. The Annals of Mathematical Statistics, 20(1), 125-126.
    .. [2] Kornbrot, D. (2014). Point biserial correlation. Wiley StatsRef: Statistics Reference Online.
    """
    x = np.asarray(x)
    y = np.asarray(y)

    try:
        check_length_are_same(x, y)
        check_if_zero_variance(x)
        check_if_zero_variance(y)
    except ValidationError as e:
        warnings.warn(str(e))
        return np.nan

    try:
        check_if_data_is_dichotomous(y)
    except ValidationError as e:
        warnings.warn(str(e))
        return np.nan

    x1 = x[y == 1]
    x0 = x[y == 0]

    M1 = np.mean(x1)
    M0 = np.mean(x0)
    s = np.std(x, ddof=0)

    p = np.mean(y)
    q = 1 - p

    return (M1 - M0) / s * np.sqrt(p * q)


def tetrachoric(x: ArrayLike[int], y: ArrayLike[int]) -> float:
    """
    Estimate the tetrachoric correlation coefficient between two dichotomous variables.

    The tetrachoric correlation assumes that the two observed dichotomous variables
    are thresholded representations of underlying continuous variables that follow
    a bivariate normal distribution.

    This method is appropriate when both variables are dichotomous (e.g., yes/no,
    success/failure) but believed to originate from an underlying normally
    distributed process.


    Parameters
    ----------
    x : array-like
        Dichotomous variable (consisting of 0 and 1).
    y : array-like
        Dichotomous variable (consisting of 0 and 1).

    Returns
    -------
    float
        Tetrachoric correlation coefficient.


    Examples
    --------
    >>> from ordinalcorr import tetrachoric
    >>> x = [0, 0, 1, 1, 1]
    >>> y = [0, 1, 0, 1, 1]
    >>> tetrachoric(x, y)

    """
    # NOTE: The estimation is the same as polyserial with dichotomous variables.
    # However, this is a bit smaller computational complexity and faster than polyserial.
    x = np.asarray(x)
    y = np.asarray(y)

    try:
        check_length_are_same(x, y)
        check_if_zero_variance(x)
        check_if_zero_variance(y)
        check_if_data_is_dichotomous(x)
        check_if_data_is_dichotomous(y)
    except ValidationError as e:
        warnings.warn(str(e))
        return np.nan

    n = len(x)
    n00 = np.sum((x == 0) & (y == 0))
    n01 = np.sum((x == 0) & (y == 1))
    n10 = np.sum((x == 1) & (y == 0))
    n11 = np.sum((x == 1) & (y == 1))

    px0 = (n00 + n01) / n
    py0 = (n00 + n10) / n

    tau_x = norm.ppf(px0)
    tau_y = norm.ppf(py0)

    def neg_log_likelihood(rho):
        phi_x = norm.cdf(tau_x)
        phi_y = norm.cdf(tau_y)

        cov = np.array([[1, rho], [rho, 1]])
        p00 = multivariate_normal.cdf([tau_x, tau_y], mean=[0, 0], cov=cov)
        p01 = phi_x - p00
        p10 = phi_y - p00
        p11 = 1 - phi_x - phi_y + p00

        probs = np.array([p00, p01, p10, p11])
        counts = np.array([n00, n01, n10, n11])

        assert np.all(probs >= 0), f"{probs=}"

        return -(counts @ np.log(probs))

    eps = 1e-10
    result = minimize_scalar(
        neg_log_likelihood, bounds=(-1 + eps, 1 - eps), method="bounded"
    )
    return result.x



import unittest
import numpy as np
import pandas as pd
from ordinalcorr.corrmatrix import (
    hetcor,
    is_cols_ordinal,
)


class TestHetcor(unittest.TestCase):

    def test_normal_data(self):
        data = pd.DataFrame(
            {
                "continuous": np.repeat([0.1, 0.2, 0.3], 10),
                "dichotomous": np.repeat([0, 1, 1], 10),
                "polytomous": np.repeat([7, 5, 3], 10),
            }
        )
        actual = hetcor(data).to_numpy()
        expect = np.array(
            [
                [1, 1, -1],
                [1, 1, -1],
                [-1, -1, 1],
            ]
        )
        self.assertTrue(np.isclose(actual, expect, atol=1e-2).all())


class TestIsColsOrdinal(unittest.TestCase):
    """test helper functions in corrmatrix"""

    def test_two_cols(self):
        data = pd.DataFrame(
            {
                "continuous": np.repeat([0.1, 0.2, 0.3], 10),
                "dichotomous": np.repeat([0, 1, 1], 10),
                "polytomous": np.repeat([7, 5, 3], 10),
            }
        )
        actual = is_cols_ordinal(data, n_unique=10)
        expected = [False, True, True]
        self.assertEqual(list(actual), expected)


if __name__ == "__main__":
    unittest.main()



import numpy as np


class ValidationError(Exception):
    pass


def check_if_zero_variance(x):
    if len(set(x)) == 1:
        raise ValidationError(
            "all elements in the input are the same and zero variance"
        )


def check_if_data_is_dichotomous(y):
    is_dichotomous = set(np.unique(y)) == {0, 1}
    if not is_dichotomous:
        raise ValidationError(
            "dichotomous (binary) variable 'y' must be consist of {0, 1}"
        )


def check_length_are_same(x, y):
    if len(x) != len(y):
        raise ValidationError("length of x and y must be the same")



import unittest
import numpy as np
from ordinalcorr.polytomous import (
    estimate_thresholds,
    normalize_ordinal,
)  # Assuming this package is installed


class TestPolytomous(unittest.TestCase):
    """test helper functions in polytomous"""

    def test_estimate_thresholds_1(self):
        x = np.tile([1, 2, 3], 10)
        tau = estimate_thresholds(x)
        self.assertTrue(np.isclose(tau, [-100.0, -0.4307273, 0.4307273, 100.0]).all())

    def test_estimate_thresholds_2(self):
        x = np.repeat([2, 4, 6], 20)
        tau = estimate_thresholds(x)
        self.assertTrue(np.isclose(tau, [-100.0, -0.4307273, 0.4307273, 100.0]).all())

    def test_normalize_ordinal_1(self):
        x = np.array([1, 2, 3, 1, 2, 3])
        actual = normalize_ordinal(x)
        expected = np.array([0, 1, 2, 0, 1, 2])
        self.assertEqual(actual.tolist(), expected.tolist())

    def test_normalize_ordinal_2(self):
        x = np.array([2, 4, 9])
        actual = normalize_ordinal(x)
        expected = np.array([0, 1, 2])
        self.assertEqual(actual.tolist(), expected.tolist())


if __name__ == "__main__":
    unittest.main()



"""
ordinalcorr - correlation coefficients for ordinal-scale variables
"""

__version__ = "0.6.1"

from ordinalcorr.polytomous import polychoric, polyserial
from ordinalcorr.dichotomous import biserial, point_biserial, tetrachoric
from ordinalcorr.corrmatrix import hetcor

__all__ = [
    "polychoric",
    "polyserial",
    "biserial",
    "point_biserial",
    "tetrachoric",
    "hetcor",
]



import unittest
import numpy as np
from ordinalcorr.validation import (
    ValidationError,
    check_if_zero_variance,
)  # Assuming this package is installed


class TestValidation(unittest.TestCase):
    """test helper functions in polytomous"""

    def test_check_if_zero_variance(self):
        # expected passing the check
        x = np.repeat([2, 4, 6], 10)
        check_if_zero_variance(x)

        # expected error
        x = np.repeat([42], 10)
        with self.assertRaises(ValidationError):
            check_if_zero_variance(x)


if __name__ == "__main__":
    unittest.main()



import numpy as np
import pandas as pd
from scipy.stats import multivariate_normal
from pathlib import Path

data_dir = Path("./data")


def generate_data(rho=0.5, bins=3):
    # Generate data by standard normal distribution
    n = 1000
    mean = [0, 0]
    std = [1, 1]
    cov = rho * std[0] * std[1]
    Cov = np.array([[std[0] ** 2, cov], [cov, std[1] ** 2]])
    X = multivariate_normal.rvs(mean=mean, cov=Cov, size=n, random_state=0)
    df = pd.DataFrame(X, columns=["x1", "x2"])
    for col in df.columns:
        df[col] = pd.cut(df[col], bins=bins, labels=range(bins)).astype(int)
    return df


if __name__ == "__main__":
    data_dir.mkdir()

    for bins in [2, 3, 5]:
        for rho in np.arange(-1, 1, 0.1):
            df = generate_data(rho=rho, bins=bins)
            df.to_csv(data_dir / f"normal_rho={rho:.2f}_bins={bins}.csv", index=False)



import numpy as np
import pandas as pd


print(
    f"""
# ----------------------------------------
# Polychoric
# ----------------------------------------
R vs Python results:

"""
)


res_r = pd.read_csv("results_r.csv")
res_py = pd.read_csv("results_python.csv")

print("R results:")
print(res_r)
print("\nPython results:")
print(res_py)


diff = res_r - res_py
print("\nDifference:")
print(diff)

print("\nisClose:")
print(diff.abs() < 0.01)



import pandas as pd
from pathlib import Path

from ordinalcorr import point_biserial
from scipy.stats import pointbiserialr


data_dir = Path("./data")
csv_paths = data_dir.glob("*.csv")

results = []
for csv_path in sorted(csv_paths):
    df = pd.read_csv(csv_path)
    rho = point_biserial(df["x"], df["y"])
    rho_scipy, _ = pointbiserialr(df["x"], df["y"])
    results.append({"file": csv_path.name, "rho": rho, "rho_scipy": rho_scipy})

pd.DataFrame(results).to_csv("results_python.csv", index=False)



import unittest
import numpy as np
from ordinalcorr import tetrachoric, polychoric


class TestTetrachoricCorr(unittest.TestCase):

    def test_positive_correlation(self):
        x = np.repeat([0, 1], 10)
        y = np.repeat([0, 1], 10)
        rho = tetrachoric(x, y)
        self.assertTrue(0.9 < rho <= 1, f"Expected high rho, got {rho}")

    def test_inverse_correlation(self):
        x = np.tile([0, 1], 10)
        y = np.tile([1, 0], 10)
        rho = tetrachoric(x, y)
        self.assertTrue(-1 <= rho < -0.9, f"Expected strong negative rho, got {rho}")

    def test_no_correlation(self):
        x = np.repeat([0, 1, 0, 1], 20)
        y = np.repeat([0, 0, 1, 1], 20)
        rho = tetrachoric(x, y)
        self.assertTrue(-0.1 < rho < 0.1, f"Expected close to zero rho, got {rho}")

    def test_single_category(self):
        x = np.repeat([1], 10)
        y = np.repeat([0], 10)
        rho = tetrachoric(x, y)
        self.assertTrue(
            np.isnan(rho),
            f"Expected undefined or near-zero rho, got {rho}",
        )

    def test_different_length(self):
        x = np.repeat([0, 1], 10)
        y = np.repeat([1, 0], 11)
        rho = tetrachoric(x, y)
        self.assertTrue(np.isnan(rho), f"Expected nan, got {rho}")

    def test_validation_for_zero_variance(self):
        x = np.repeat([0], 10)
        y = np.repeat([1], 10)
        rho = tetrachoric(x, y)
        self.assertTrue(
            np.isnan(rho),
            f"Expected undefined or near-zero rho, got {rho}",
        )

    def test_validation_for_polytomous_variables(self):
        x = np.repeat([0, 1, 2], 10)
        y = np.repeat([0, 1, 0], 10)
        rho = tetrachoric(x, y)
        self.assertTrue(
            np.isnan(rho),
            f"Expected undefined or near-zero rho, got {rho}",
        )

    def test_tetrachoric_polychoric_equivalence(self):
        test_cases = [
            (np.repeat([0, 1], 10), np.repeat([0, 1], 10)),
            (np.repeat([0, 1], 10), np.repeat([1, 0], 10)),
            (np.repeat([1, 0], 10), np.repeat([0, 1], 10)),
            (np.repeat([1, 0], 10), np.repeat([1, 0], 10)),
        ]
        for x, y in test_cases:
            with self.subTest(x=x, y=y):
                rho_tetrachoric = tetrachoric(x, y)
                rho_polychoric = polychoric(x, y)
                self.assertAlmostEqual(rho_tetrachoric, rho_polychoric)


if __name__ == "__main__":
    unittest.main()



import unittest
import numpy as np
import pandas as pd
from scipy.stats import multivariate_normal
from ordinalcorr import polychoric  # Assuming this package is installed


class TestPolychoricCorr(unittest.TestCase):

    def test_known_result(self):
        x = np.repeat([1, 2, 3], 10)
        y = np.repeat([1, 2, 3], 10)
        rho = polychoric(x, y)
        self.assertTrue(0.9 < rho < 1.0, f"Expected high rho, got {rho}")

    def test_inverse_correlation(self):
        x = np.tile([1, 2, 3], 10)
        y = np.tile([6, 4, 2], 10)
        rho = polychoric(x, y)
        self.assertTrue(-1.0 < rho < -0.9, f"Expected strong negative rho, got {rho}")

    def test_no_correlation(self):
        x = np.tile([1, 2, 3], 20)
        y = np.repeat([1, 2, 3], 20)
        rho = polychoric(x, y)
        self.assertTrue(-0.1 < rho < 0.1, f"Expected close to zero rho, got {rho}")

    def test_single_category(self):
        x = np.repeat([1], 10)
        y = np.repeat([0], 10)
        rho = polychoric(x, y)
        self.assertTrue(
            np.isnan(rho) or abs(rho) < 1e-6,
            f"Expected undefined or near-zero rho, got {rho}",
        )

    def test_different_length(self):
        x = np.repeat([0, 1, 2], 10)
        y = np.repeat([2, 0, 1], 11)
        rho = polychoric(x, y)
        self.assertTrue(np.isnan(rho), f"Expected nan, got {rho}")

    def test_different_rho(self):
        for bins in [2, 3]:
            for rho in np.arange(-1, 1, 0.1):
                # change tolerance because estimation error is large for small rho
                tolerance = 0.3 if rho < 0.5 else 0.1

                df = gen_data_for_polychoric(rho=rho, bins=bins, size=1000)
                rho_hat = polychoric(df["x1"], df["x2"])
                # print(f"{bins=} {rho=:>.3f} {rho_hat=:>.3f}")
                self.assertTrue(
                    np.isclose(rho, rho_hat, atol=tolerance),
                    f"Expected rho close to {rho}, got {rho_hat}",
                )


def gen_data_for_polychoric(rho=0.5, bins=3, size=1000):
    # Generate data by standard normal distribution
    mean = [0, 0]
    std = [1, 1]
    cov = rho * std[0] * std[1]
    Cov = np.array([[std[0] ** 2, cov], [cov, std[1] ** 2]])
    X = multivariate_normal.rvs(mean=mean, cov=Cov, size=size, random_state=0)
    df = pd.DataFrame(X, columns=["x1", "x2"])
    for col in df.columns:
        df[col] = pd.cut(df[col], bins=bins, labels=range(bins)).astype(int)
    return df


if __name__ == "__main__":
    unittest.main()



import unittest
import numpy as np
from ordinalcorr import biserial  # Assuming this package is installed


class TestBiserialCorr(unittest.TestCase):

    def test_known_result(self):
        x = np.repeat([0.1, 0.2, 0.3, 0.4, 0.5], 10)
        y = np.repeat([0, 0, 1, 1, 1], 10)
        rho = biserial(x, y)
        self.assertTrue(0.9 < rho, f"Expected high rho, got {rho}")

    def test_inverse_correlation(self):
        x = np.tile([1, 2, 3, 4, 5], 10)
        y = np.tile([1, 1, 0, 0, 0], 10)
        rho = biserial(x, y)
        self.assertTrue(rho < -0.9, f"Expected strong negative rho, got {rho}")

    def test_no_correlation(self):
        x = np.tile([1, 2, 3], 20)
        y = np.repeat([0, 1, 0], 20)
        rho = biserial(x, y)
        self.assertTrue(-0.1 < rho < 0.1, f"Expected close to zero rho, got {rho}")

    def test_single_category(self):
        x = np.repeat([1], 10)
        y = np.repeat([0], 10)
        rho = biserial(x, y)
        self.assertTrue(
            np.isnan(rho) or abs(rho) < 1e-6,
            f"Expected undefined or near-zero rho, got {rho}",
        )

    def test_different_length(self):
        x = np.repeat([0, 1], 10)
        y = np.repeat([1, 0], 11)
        rho = biserial(x, y)
        self.assertTrue(np.isnan(rho), f"Expected nan, got {rho}")


if __name__ == "__main__":
    unittest.main()
